from twisted.internet import defer, task, reactor, protocol, error
import time
import re

_emoticon_regex = re.compile(
    r'(?:^|\s+)(>?[:;x][D3P(<To]|[DT3o][:;x]<?|'
    r'[;.ox^<>-](?:~|_+)?[;.ox^<>-];*)\s*$', re.I)

ISOFORMAT = '%Y-%m-%dT%H:%M:%S'

def parallel(iterable, count, f, *args, **named):
    coop = task.Cooperator()
    work = (f(elem, *args, **named) for elem in iterable)
    return defer.DeferredList([coop.coiterate(work) for i in xrange(count)])

def time_deferred(d):
    def _cb(x):
        return time.time() - now, x
    d.addCallback(_cb)
    now = time.time()
    return d

class _BzrRevnoProtocol(protocol.ProcessProtocol):
    def __init__(self):
        self.deferred = defer.Deferred()
        self.buffer = ''

    def childDataReceived(self, childFD, data):
        assert childFD == 1
        self.buffer += data

    def processEnded(self, status):
        if status.value.exitCode == 0:
            try:
                ret = int(self.buffer.strip())
            except ValueError:
                self.deferred.errback()
            else:
                self.deferred.callback(ret)
        else:
            self.deferred.errback(status)

def bzr_revno():
    prot = _BzrRevnoProtocol()
    command = ['bzr', 'revno', __file__]
    print command
    reactor.spawnProcess(prot, command[0], command,
        env=None, childFDs={1: 'r'})
    return prot.deferred
